import 'package:binarize/binarize.dart';

/// {@template raw_list}
/// Reads and write a raw list of [type].
///
/// It has a byte length of `list.fold(0, (p, e) => p + type.length(e))`.
///
/// Usage:
///
/// ```dart
/// payload.set(RawList(string8, amount: 3), ['1', '2', '3']);
/// ```
/// {@endtemplate}
class RawList<T> extends PayloadType<List<T>> {
  /// {@macro raw_list}
  const RawList(this.type, {required this.amount});

  /// The amount of values this type supports.
  final int amount;

  /// The payload type to use.
  final PayloadType<T> type;

  @override
  int length(List<T> value) {
    var currentOffset = 0;
    for (var i = 0; i < amount; i++) {
      final item = value[i];
      currentOffset += type.length(item);
    }
    return currentOffset;
  }

  @override
  List<T> get(ByteData data, int offset) {
    var currentOffset = offset;
    final result = <T>[];
    for (var i = 0; i < amount; i++) {
      result.add(type.get(data, currentOffset));
      currentOffset += type.length(result.last);
    }
    return result;
  }

  @override
  void set(List<T> value, ByteData data, int offset) {
    var currentOffset = offset;
    for (var i = 0; i < amount; i++) {
      final item = value[i];
      type.set(item, data, currentOffset);
      currentOffset += type.length(item);
    }
  }
}
